Package: gWidgets
Version: 0.0-54
Title: gWidgets API for building toolkit-independent, interactive GUIs
Author: John Verzani. Based on  the iwidgets code of Simon Urbanek and suggestions by Simon Urbanek, Philippe Grosjean and Michael Lawrence
Maintainer: John Verzani <jverzani@gmail.com>
Depends: methods, utils
Suggests: gWidgetstcltk
Description: gWidgets provides a toolkit-independent API for building interactive GUIs. At least one of the 'gWidgetsXXX packages', such as gWidgetstcltk, needs to be installed. Some icons are on loan from the scigraphica project http://scigraphica.sourceforge.net.
License: GPL (>= 2)
URL: https://r-forge.r-project.org/R/?group_id=761
Collate: aaaGenerics.R common.R toolkitClasses.R guiWidget.R
        guiContainer.R guiComponent.R guiComponentWithItems.R
        handlers.R ggroup.R gframe.R gexpandgroup.R gspinbutton.R
        gslider.R gsvg.R ggenericwidget.R gtable.R ggraphics.R gtext.R
        dialogs.R gtoolbar.R gaction.R ghelp.R gtree.R gbutton.R
        ghtml.R gcalendar.R gimage.R gcheckbox.R glabel.R
        gcheckboxgroup.R glayout.R gcombobox.R gmenu.R gvarbrowser.R
        gcommandline.R gnotebook.R gvariables.R gdf.R gpanedgroup.R
        gwindow.R gdfnotebook.R gradio.R gedit.R gseparator.R icons.R
        gfile.R gformlayout.R gstatusbar.R bbbGenericsANY.R zzz.R
LazyLoad: yes
Packaged: 2014-08-27 20:53:43 UTC; verzani
NeedsCompilation: no
Repository: CRAN
Date/Publication: 2014-08-28 00:18:31
Built: R 3.4.1; ; 2018-02-04 04:35:56 UTC; windows
