Icons for gWidgets. 

Some were taken from the scigraphica project at
sourceforge.  Some were manipulated from Dropline Nuovo!
http://art.gnome.org/themes/icon/1112. 

Due to tcltk restrictions, these should be gif files.
